#include "shared.h"
#include <cmath>
#include "analysisFunction.h"


class ansErr:public general{
public:
  int doAnsError;
  int nInd;
  int minQ;
  int minMapQ;
  //none optional stuff
  FILE *outfile;
  FILE *outfile2;
  ansErr(const char *outfiles,argStruct *arguments,int inputtype);
  ~ansErr();
  void getOptions(argStruct *arguments);
  void run(funkyPars  *pars);
  void print(funkyPars *pars);
  void clean(funkyPars *pars);
  void printArg(FILE *argFile);

  size_t **alleleCounts; //[ind][125]; 
};

void ansErr::printArg(FILE *argFile){
  fprintf(argFile,"%s:\n\n",__FILE__);
  fprintf(argFile,"-doAnsError\t%d\n",doAnsError);
  fprintf(argFile,"\t->1: EM v1\n");
  fprintf(argFile,"\t->2: EM v2\n");
  fprintf(argFile,"-minQ\t%d\n",minQ);
  fprintf(argFile,"-minMapQ\t%d\n",minMapQ);
  //  fprintf(argFile,"-nInd\t%d\n",nInd);
  fprintf(argFile,"\n");
}

void ansErr::getOptions(argStruct *arguments){
  //default
  doAnsError=0;
  //from command line
  doAnsError=angsd::getArg("-doAnsError",doAnsError,arguments);
  if(doAnsError==-999){
    doAnsError=0;
    printArg(stdout);
    exit(0);
 
  }

  if(doAnsError==0)
    return;
  minQ=0;
  minQ=angsd::getArg("-minQ",minQ,arguments);
 
  minMapQ=0;
  minMapQ=angsd::getArg("-minMapQ",minMapQ,arguments);


  //  nInd=angsd::getArg("-nInd",nInd,arguments);

  nInd=arguments->nInd;
  //  fprintf(stderr,"Estimating ancestral errors Nind: %d\n",nInd);
  //  exit(0);
  printArg(arguments->argumentFile);

}

ansErr::ansErr(const char *outfiles,argStruct *arguments,int inputtype){


 getOptions(arguments);

  if(doAnsError==0)
    return;

  //inputtype 0) soap 1) samglf 2) samglfClean 3) tglf 4)sim type 5) beagle
  if(inputtype!=0&&inputtype!=7){
    fprintf(stderr,"Error: bam or soap input needed for -doAnsError \n");
    exit(0);
  }
 
  fprintf(stderr,"Estimating ancestral errors: %s\n",outfiles);
  //make output files
  const char* postfix;
  postfix=".ancError";
  outfile = openFile(outfiles,postfix);
  const char* postfix2;
  postfix2=".ancErrorChunk";
  outfile2 = openFile(outfiles,postfix2);

  //allocate allele counts
  alleleCounts = new size_t *[nInd];
  for(int i=0;i<nInd;i++)
    alleleCounts[i] = new size_t [256];
  for(int i=0;i<nInd;i++)
    for(int j=0;j<256;j++)
      alleleCounts[i][j]=0;

}


ansErr::~ansErr(){
  //  fprintf(stderr,"[%s] calling destructor\n",__FUNCTION__);
  //  fflush(stderr);
  if(doAnsError==0)
    return;

  if(doAnsError==2){
    for(int i=0;i<nInd;i++){
      for(int j=0;j<125;j++)
	fprintf(outfile,"%lu\t",alleleCounts[i][j]);
      fprintf(outfile,"\n");
    }
  }


  for(int i=0;i<nInd;i++)
    delete[]  alleleCounts[i];
  delete [] alleleCounts; 

  fclose(outfile);
}


void ansErr::clean(funkyPars *pars){


  if(doAnsError==0)
    return;

 

}



void ansErr::print(funkyPars *pars){


  if(doAnsError==0)
    return;

  if(doAnsError==1){
    for(int s=0;s<pars->numSites;s++){
      fprintf(outfile,"%c",intToRef[pars->anc[s]]);
      fprintf(outfile,"\t%c",intToRef[pars->ref[s]]);

     for(int i=0;i<pars->nInd;i++){
	
	//      tNode &nd = pars->chk->nd[s][i];
	int allele=4;
	for(int j=0;j<pars->chk->nd[s][i].l;j++){
	  if(pars->chk->nd[s][i].qs[j]>=minQ && pars->chk->nd[s][i].mapQ[j]>=minMapQ){
	     allele=refToInt[pars->chk->nd[s][i].seq[j]];
	    break;
	  }
	}

	fprintf(outfile,"\t%c",intToRef[allele]);
      }
      fprintf(outfile,"\n");
    }
  }
  else if(doAnsError==2){
   
    for(int s=0;s<pars->numSites;s++){
      for(int i=0;i<pars->nInd;i++){

	/*
	if(pars->chk->nd[s][i].l>0){
	  //  int hej = pars->anc[s]*5+pars->chk->nd[s][i].seq[0];
	  //fprintf(outfile,"\t%d",hej);
	  //alleleCounts[i][0]++;


	  //	   alleleCounts[i][pars->ref[s]*5+refToInt[pars->chk->nd[s][i].seq[0]]]++;

	   alleleCounts[i][pars->anc[s]*25+pars->ref[s]*5+refToInt[pars->chk->nd[s][i].seq[0]]]++;
	}
	*/

	for(int j=0;j<pars->chk->nd[s][i].l;j++){
	  if(pars->chk->nd[s][i].qs[j]>=minQ && pars->chk->nd[s][i].mapQ[j]>=minMapQ){
	    alleleCounts[i][pars->anc[s]*25+pars->ref[s]*5+refToInt[pars->chk->nd[s][i].seq[j]]]++;
	    break;
	  }
	}


      }
      //            fprintf(outfile,"\n");
    }


  }


  if(doAnsError==2){
    fprintf(outfile2,"Start \n");
    for(int i=0;i<nInd;i++){
      for(int j=0;j<125;j++)
	fprintf(outfile2,"%lu\t",alleleCounts[i][j]);
      fprintf(outfile2,"\n");
    }
    fprintf(outfile2,"Stop \n");
  }


}


void ansErr::run(funkyPars *pars){

  if(doAnsError==0)
    return;




}

