#include <vector>
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include "parseArgs_bambi.h"
#include "shared.h"
#include "analysisFunction.h"

//below is extern from shared.cpp
extern int maxThreads;

// below is default samtools parameters
int uniqueOnly = 0;
int only_proper_pairs = 1;
int remove_bads = 1;

int rute = 0;



void printArg(FILE *argFile,args *ret){
  fprintf(argFile,"%s: bam reader:\n",__FILE__);
  fprintf(argFile,"-bam\t%s\n",ret->inputfile);
  fprintf(argFile,"-r\t%s\n",ret->region);
  fprintf(argFile,"-i\t%s\n",ret->inputfile);
  fprintf(argFile,"-rf\t%s\n",ret->regionfile);
  fprintf(argFile,"-remove_bads\t%d\n",remove_bads);
  fprintf(argFile,"-nInd\t%d\n",ret->nInd);
  fprintf(argFile,"-nLines\t%d\n",ret->nLines);
  fprintf(argFile,"-type\t%d\n",ret->type);
  fprintf(argFile,"-uniqueOnly\t%d\n",ret->uniqueOnly);
  fprintf(argFile,"-show\t%d\n",ret->show);
  fprintf(argFile,"-nThreads\t%d\n",ret->nThreads);
  fprintf(argFile,"-only_proper_pairs\t%d\n",only_proper_pairs);
  fprintf(argFile,"\n");

}


//read program parameters
args *getArgsBambi(argStruct *arguments){
  args *ret = new args;
  ret->inputfile = NULL;
  ret->inputfiles = NULL;
  ret->region = NULL;
  ret->nThreads = maxThreads;
  ret->callback= NULL;
  ret->show = 0;
  ret->uniqueOnly = 0;
  ret->nInd = 0;
  ret->type = 1;
  ret->nLines = 50;
  ret->refName = NULL;
  ret->inputfile=ret->inputfiles=ret->region=ret->regionfile=NULL;
  ret->jobtype = 2;

  ret->inputfiles=angsd::getArg("-bam",ret->inputfiles,arguments);
  ret->inputfile=angsd::getArg("-i",ret->inputfile,arguments);

  if(ret->inputfiles==NULL && ret->inputfile==NULL)
    return NULL;
  if(ret->inputfiles!=NULL&&strcmp(ret->inputfiles,"-999")==0){//DRAGON fix this
    ret->inputfiles=NULL;
    printArg(stdout,ret);
    exit(0);
  }
  


  ret->region =angsd::getArg("-r",ret->region,arguments);
  only_proper_pairs =angsd::getArg("-only_proper_pairs",only_proper_pairs,arguments);
  ret->regionfile = angsd::getArg("-rf",ret->regionfile,arguments);
  remove_bads = angsd::getArg("-remove_bads",remove_bads,arguments);
  ret->nInd = angsd::getArg("-nInd",ret->nInd,arguments);
  ret->nLines = angsd::getArg("-nLines",ret->nLines,arguments);
  ret->type = angsd::getArg("-type",ret->type,arguments);
  ret->uniqueOnly = angsd::getArg("-uniqueOnly",ret->uniqueOnly,arguments);
  ret->show = angsd::getArg("-show",ret->show,arguments);
  ret->nThreads = angsd::getArg("-nThreads",ret->nThreads,arguments);
  ret->jobtype= angsd::getArg("-jobtype",ret->jobtype,arguments);
  ret->refName = angsd::getArg("-ref",ret->refName,arguments);
  rute = angsd::getArg("-rute",rute,arguments);
  if(ret->inputfile && ret->inputfile[0]=='-'){
    fprintf(stderr,"[Error] Problem understanding inputfilename: %s\n",ret->inputfile);
    return NULL;
  }
  uniqueOnly = ret->uniqueOnly;
  printArg(arguments->argumentFile,ret);
  return ret;
}

void print(args *a,FILE *fp){
  fprintf(fp,"\njobtype=%d, 1=view, 2=uppile\n",a->jobtype);
  fprintf(fp,"type=%d, 0=slow/secure 1=fast/insecure\n",a->type);
  fprintf(fp,"inputfile=%s (single sample)\n",a->inputfile);
  fprintf(fp,"inputfiles=%s (multisample)\n",a->inputfiles);
  fprintf(fp,"region=%s (single region)\n",a->region);
  fprintf(fp,"nInd=%d (single region)\n",a->nInd);
  fprintf(fp,"regionfile=%s (multiregion)\n\n",a->regionfile);
  //  fprintf(fp,"nLines=%d\tnThreads=%d\n",a->nLines,a->nThreads);
  fprintf(fp,"nLines=%d\n",a->nLines);
  fprintf(fp,"-GL INT: -1=mpileup 0=soap 1=samtools 2=gatk\n");
}

 void bamInfo(FILE *fp){
   fprintf(fp,"single sample)\n");
   fprintf(fp,"\t1) prog TYPE .bam\n");
   fprintf(fp,"\t2) prog TYPE .bam -r REGION\n");
   fprintf(fp,"\t3) prog TYPE .bam -rf REGIONFILE chr\n");
   fprintf(fp,"multisample)\n");
   fprintf(fp,"\t4) prog TYPE -b BAMLIST \n");
   fprintf(fp,"\t5) prog TYPE -b BAMLIST -r REGION\n");
   fprintf(fp,"\t6) prog TYPE -b BAMLIST -r REGIONFILE\n");
   fprintf(fp,"TYPE=view,uppile\n");
   fprintf(fp,"REGION:\n");
   fprintf(fp,"\tchr\n");
   fprintf(fp,"\tchr:start\n");
   fprintf(fp,"\tchr:-stop\n");
   fprintf(fp,"\tchr:start-stop\n");
   fprintf(fp,"INFO:\n\tBAMLIST is filelist containing the bamfiles\n");
   fprintf(fp,"\tREGIONFILE  is filelist containing the regions\n");
   //   fprintf(fp,"General:\n\t-type INT, INT=0 slow, INT=1 fast\n");
   fprintf(fp,"GENOTYPELIKELIHOODS\n");
   fprintf(fp,"\t for uppile GL can be estimate with -GL INT\n");
   fprintf(fp,"\t -1=mpileup(no GL) 0=soap 1=samtools 2=gatk\n");
   fprintf(fp,"EXAMPLES:\n");
   fprintf(fp,"\tsamtools LK for single bam\n");
   fprintf(fp,"\t./bammer uppile -GL 1 tmp.bam\n\n");
   fprintf(fp,"samtools LK for 2 first bam in file list\n");
   fprintf(fp,"\n./bammer uppile -GL 1 tmp.bam\n");
   
   
 }
